
;(function ($, window, document, undefined) {


//	=========================================================================================================
// 		Create the defaults once
//	=========================================================================================================


	var pluginName = 'upload',
		defaults = {
		
			'name' : 'images',								// Input name
			'multiple': true,								// True to allow multiple upload
			'dir': 'files',									// Dir where save the files
			'url': 'save.php',								// File php
			'data': {},										// Data to append at the ajac call
			
			'button': 'Select files',						// Text of the button
			'buttonAbort': 'Abort',							// Text of the button
			'titleButton': 'Select files',					// Title of the button
			'titleButtonAbort': 'Abort',					// Title of the button
			
			'accept' : ['image/jpeg', 'image/png'],			// MIME type accepted
			'minSize': 0,									// Min Size
			'maxSize': 10000000,							// Max Size
			'imgWidth': 0,									// Image Width
			'imgHeight': 0,									// Image Height
			'dayClearTmp': 5,								// Delete files older than the indicated days in the folder, 0 annuls				
			
			'complete': function() {},						// complete function
			
			'messages': function(key, filename) {
			
				switch (key) { 
				
					case 'type': 
						alert("Error file '" + filename + "': File type forbidden");
					break; 

					case 'minSize': 
						alert("Error file '" + filename + "': File size");
					break; 
					
					case 'maxSize': 
						alert("Error file '" + filename + "': File size");
					break;
					
					case 'dim': 
						alert("Error file '" + filename + "': File dimension forbidden. Dimension accepted " + this.imgWidth + 'px X ' + this.imgHeight + 'px');
					break; 
					
					case 'noUpload': 
						alert("Can not upload");
					break;
					
				}
				
			}
		};

//	=========================================================================================================		
//		Constructor
//	=========================================================================================================

	var Plugin = function(el, options) {

		this.el = el;
		this.options = $.extend({}, defaults, options) ;

		this._defaults = defaults;
		this._name = pluginName;

		this.init();

	}
	
	
//	=========================================================================================================		
//		Plugin
//	=========================================================================================================


	$.extend(Plugin.prototype, {
		
	//	Initialize
	//	-----------------------------------------------------------------------------------------------------
		
		init:			function () {
							
							this.input = this.getInput(),
							this.form = this.getForm(),
							this.button = this.getButton(),
							this.buttonAbort = this.getButtonAbort();
							
							this.form.append(this.input);
							$(this.el).append(this.form).append(this.button).append(this.buttonAbort);
							
							this.input.change($.proxy(function(e) {
								
								this.upload(e.currentTarget);
								
							}, this));
							
						},
		
	//	Upload file after input change
	//	-----------------------------------------------------------------------------------------------------
	
		upload:			function($input) {
		
							var len = $input.files.length,
								file,
								tmpPromises = [],
								files = [];
							
							this.num = 0;
							
							for (i = 0; i < len; i++) {	
								
								(function(App, file, tmpPromise) {
									
									var save = function(file) {
										App.num ++;
										files.push(file);
									}
									
									if (App.checkFile(file)) {
										
										if (file.type.match(/image.*/)) {
										
											App.readImg(file, function(img) {										
												if (App.checkDim(img)) save(file);
												else App.options.messages('dim', file.name);
												tmpPromise.resolve();													
											});
											
										} else {
										
											save(file);
											tmpPromise.resolve();
											
										}
									
									} else {
										
										tmpPromise.resolve();
									
									}
									
									
								})(this, $input.files[i], tmpPromises[i] = $.Deferred());
							
							}
							
							$.when.apply($, tmpPromises).done($.proxy(function() {
							
								if (this.num > 0) this.options.complete(files);
								
							}, this));
							
						},
		
	//	Get the input element
	//	-----------------------------------------------------------------------------------------------------
	
		getInput:		function() {
		
							var $input = $('<input />', {
								'type' : 'file',
								'name' : this.options.name + '[]',
								'accept' : this.options.accept,
								'multiple': this.options.multiple
							});
										
							$input.bind('empty', function(e) {
								$input.val('');
							});
							
							return $input;
		
						},
						
	//	Get the form element
	//	-----------------------------------------------------------------------------------------------------
	
		getForm:		function() {
		
							$form = $('<form />', {
								'method' : 'post',
								'enctype': 'multipart/form-data'
							}).css({'position': 'absolute', 'left': '-5000px', 'top': '-5000px'});
							
							return $form;
		
						},
						
	//	Get the Button Upload
	//	-----------------------------------------------------------------------------------------------------
	
		getButton:		function() {
		
							var $ico = $('<span />').addClass('ico'),
								$label = $('<span />', {
									'text': this.options.button,
									'title': this.options.titleButton
								}).addClass('label'),
								$contentButton = $('<span />').addClass('content-button').append($ico).append($label),
								$button = $('<span />').addClass('button ha orange select-files').append($contentButton);
							
							$button.bind('click', $.proxy(function(e) {
								e.preventDefault();
								if ($button.data('disabled') !== true) this.input.trigger('click');
							}, this));
							
							$button.bind('enableButton', function(e) {
								$button.show().removeData('disabled').removeClass('disabled');
							});
							
							$button.bind('disableButton', function(e) {
								$button.hide().data('disabled', true).addClass('disabled');
							});
							
							return $button;
		
						},	
						
	//	Get the Button Abort
	//	-----------------------------------------------------------------------------------------------------
	
		getButtonAbort:	function() {
			
							var $buttonAbort = $('<button />', {
								'text': this.options.buttonAbort,
								'title': this.options.titleButtonAbort
							}).addClass('abort').hide();
							
							$buttonAbort.bind('enableButton', $.proxy(function(e) {
							
								$buttonAbort.show().bind('click', $.proxy(function(e) {
									
									e.preventDefault();
									
									this.xhr.abort();
									this.input.trigger('empty');					// empty input		
									this.button.trigger('enableButton');			// enable button		
									$buttonAbort.trigger('disableButton');			// disable buttonAbort
									
									this.options.complete();
									
								}, this));
								
							}, this));
							
							$buttonAbort.bind('disableButton', function(e) {
								$buttonAbort.hide().unbind('click');
							});	
							
							return $buttonAbort;
		
						},
						
	//	Check File
	//	-----------------------------------------------------------------------------------------------------	

		checkFile:		function(file) {
		
							if (!this.checkMimeType(file.type)) this.options.messages('type', file.name);				// Check file type
							else if (!this.checkMinSize(file.size)) this.options.messages('minSize', file.name);		// Check min size
							else if (!this.checkMaxSize(file.size)) this.options.messages('maxSize', file.name);		// Check max size
							else return true;
		
						},
												
	//	Read img details
	//	-----------------------------------------------------------------------------------------------------	

		readImg:		function(file, callback) {
		
							var	reader = new FileReader(),
							img = new Image; 
								
							reader.onloadend = function (e) {  								
								img.onload = function(e) {
									callback(img);
								}								
								img.src = reader.result;								
							};
							
							reader.readAsDataURL(file);
							
						},
		
	//	Check mime type
	//	-----------------------------------------------------------------------------------------------------	
	
		checkMimeType:	function(mimeType) {
	
							if (this.options.accept.length && !this.inArray(mimeType, this.options.accept)) return false;
							else return true;
	
						},	
						
	//	Check min Size
	//	-----------------------------------------------------------------------------------------------------	
	
		checkMinSize:	function(size) {
	
							if (size < this.options.minSize || size == 0) return false;
							else return true;
	
						},	
						
	//	Check Max Size
	//	-----------------------------------------------------------------------------------------------------	
	
		checkMaxSize:	function(size) {
	
							if (size > this.options.maxSize) return false;
							else return true;
	
						},
						
	//	Check Max Size
	//	-----------------------------------------------------------------------------------------------------	
	
		checkDim:		function(img) {
	
							if (this.options.imgWidth !== 0 && img.width < this.options.imgWidth) return false;
							if (this.options.imgHeight !== 0 && img.height < this.options.imgHeight) return false;
							else return true;
	
						},
		
	//	Function: check if a element is in an array	
	//	-----------------------------------------------------------------------------------------------------	
	
		inArray:		function(needle, haystack) {
		
							var length = haystack.length;
							for(var i = 0; i < length; i++) {
								if(haystack[i] == needle) return true;
							}
							return false;
							
						}			
		
	});  

	
//	=========================================================================================================		
//		jQuery extend
//	=========================================================================================================


	$.fn[pluginName] = function (options) {
	
		return this.each(function () {
			
			if (!$.data(this, 'plugin_' + pluginName)) {
			
				$.data(this, 'plugin_' + pluginName, new Plugin(this, options));
				
			}
			
		});
		
	}

})(jQuery, window, document);