/*!
 * jQuery Tagyourimages - for jQuery 1.7+
 *
 * Copyright (c) 2012, Cadamuro Lorenzo http://tagyourimages.net
 * Released under CodeCanyon Regular License.
 * http://codecanyon.net/licenses/regular_extended
 */
 
;(function ($, window, document, undefined) {


//	=========================================================================================================
// 		Create the defaults once
//	=========================================================================================================


	var pluginName = 'tagyourimages',
		defaults = {
		
			markerSize: 30,
			marginTooltip: 10,
			markerStartOpacity: 0.4,
			markerHide: true,
			speedShowTooltip: 210,
			zIndex: 200,
			data: {}
		
		},
		classes = {
		
			wrapper:		'tagyourimages-wrap',
			markers:		'tagyourimages-markers',
			marker:			'tagyourimages-marker',
			tooltipWrap:	'tagyourimages-tooltip-wrap',
			tooltip:		'tagyourimages-tooltip',
			tooltipArrow:	'tagyourimages-tooltip-arrow',
			tooltipImage:	'tagyourimages-tooltip-image',
			tooltipLink:	'tagyourimages-tooltip-link'
		
		}

		
//	=========================================================================================================		
//		Constructor
//	=========================================================================================================


	var Plugin = function(elm, options) {

		this.el = elm;
		this.$el = $(elm);
		this.options = $.extend({}, defaults, options) ;
		this.classes = classes;
		
		this._defaults = defaults;
		this._name = pluginName;
		
		this.detectTouch();
		
		// check
		this.options.markerSize = this.options.markerSize < this._defaults.markerSize ? this._defaults.markerSize : this.options.markerSize;
		this.options.markerHide = this.isTouch ? false : this.options.markerHide;
		this.options.markerStartOpacity = this.checkOpacity() ? this.options.markerStartOpacity > 1 ? this.options.markerStartOpacity / 100 : this.options.markerStartOpacity : 0;
		
		$('html').on('mouseenter', function() {}).on('mouseleave', function() {});
		
		this.init();

	}
	
	
//	=========================================================================================================		
//		Plugin
//	=========================================================================================================


	$.extend(Plugin.prototype, {
		
		init:			function () {
							
							if (!this.$el.is('img')) return;
							
							this.id = this.getElmId();
							this.data = this.options.data['imageTag_' + this.id] ? this.options.data['imageTag_' + this.id] : false;
							
							this.waitImage(this.$el.attr('src'), $.proxy(function() {
								
								this.initElm();
								if (this.data !== false) this.renderMarkers();
								
							}, this));
							
						},
		
		getElmId:		function() {
		
							return this.$el.data('tag') ? this.$el.data('tag') : null;
		
						},
		
		initElm:		function() {
		
							this.setElmDim();
							
							var styleWrapper = $.extend(this.getElmStyle(), this.dim),
								styleResetter = {
									verticalAlign: 'middle',
									clear: 'both',
									position: 'relative',
									margin: 0,
									padding: 0,
									zIndex: 100
								},
								styleMarkers = {
									position: 'absolute',
									top: 0,
									left: 0,
									width: '100%',
									height: '100%',
									zIndex: 200
								}
							
							var $wrapper = $('<div />').addClass(this.classes.wrapper).css(styleWrapper);
							
							this.$markers = $('<div />').addClass(this.classes.markers).css(styleMarkers).css('opacity', this.options.markerHide ? this.options.markerStartOpacity : 1);		
							this.$el.wrap($wrapper).css(styleResetter).after(this.$markers);
							this.$wrapper = this.$el.parent('.' + this.classes.wrapper);
							
							if (this.options.markerHide === true) {
								
								this.$wrapper.on('mouseenter', $.proxy(function() {
									
									this.$markers.css('opacity', 1);
									this.$wrapper.css('z-index', this.options.zIndex + 100);
									
								}, this))
								
								this.$wrapper.on('mouseleave', $.proxy(function(e) {
									
									this.$markers.css('opacity', this.options.markerStartOpacity);
									this.$wrapper.css('z-index', this.options.zIndex);
									
								}, this))
								
							}
		
						},
		
		checkBoxShadow:	function() {
		
							var check = document.createElement('div'),
								shadow = !!(0 + check.style['boxShadow']);
							
							if (shadow) return 'box-shadow';
		
						},
		
		checkOpacity:	function() {
		
							var check = document.createElement('div'),
								opacity = !!(0 + check.style['opacity']);
							
							return opacity ? true : false;
		
						},
		
		waitImage:		function(src, callback) {
			
							var img = new Image();
							
							img.onload = callback;
							
							img.src = src;
		
						},
		
		detectTouch:	function() {
		
							if (!!('ontouchstart' in window)) this.isTouch = true;
							else this.isTouch = false;
		
						},
		
		getElmStyle:	function() {
		
							var style = {						
									float: this.$el.css('float'),
									margin: this.$el.css('margin'),
									padding: this.$el.css('padding'),
									position: this.$el.css('position') === 'relative' || this.$el.css('position') === 'absolute' || this.$el.css('position') === 'fixed' ? this.$el.css('position') : 'relative',
									top: this.$el.css('top'),
									left: this.$el.css('left'),
									right: this.$el.css('right'),
									bottom: this.$el.css('bottom'),
									zIndex: this.$el.css('z-index') || this.options.zIndex
								}
							
							return style;
		
						},
		
		setElmDim:		function() {
		
							this.dim = {
								
								width: this.$el.width(),
								height: this.$el.height()
								
							}
		
						},
		
		computeFactors:	function() {
		
							this.factors = {};
							this.factors.y = this.dim.height / 521234894;
							this.factors.x = this.dim.width / 521234894;
		
						},
		
		renderMarkers:	function() {
		
							setTimeout($.proxy(function() {
							
								this.computeFactors();
								
								$.each(this.data, $.proxy(function(index, val) {
								
									var $marker = this.getMarker(val).appendTo(this.$markers).data('data', val);
									
									if (this.isTouch) {
										
										$marker.on('click', $.proxy(function(e) {		
											
											if ($marker.data('active') === true) return;
											
											this.showTooltip(e);
											
											setTimeout($.proxy(function() {
												
												$('html').on('touchend.' + this.id + $marker.index() + ' touchmove', $.proxy(function(e) {		
													if (e.type == 'touchmove') {
														$('html').data('stop', true);
														return;
													} else if (e.type == 'touchend' && $('html').data('stop') == true) {
														$('html').data('stop', false);
														return;
													}
													var $target = $(e.target),
														$tooltip = $marker.data('tooltip') || $();
													if ($target.is($tooltip) || $tooltip.has($target).is($tooltip)) return;
													this.hideTooltip($marker);
												}, this));
												
											}, this));		
											
										}, this));
										
									} else {
										
										$marker.on('mouseover', $.proxy(this.showTooltip, this));
										$marker.on('mouseout', $.proxy(this.markerOut, this));
										
									}
								
								}, this));
							
							}, this));
		
						},
		
		getMarker:	function(tag) {
		
							var topPosition = tag.cordinates.y * this.factors.y,
								leftPosition = tag.cordinates.x * this.factors.x,
								shift = Math.round(this.options.markerSize / 2);
							
							var styleMarker = {
									position: 'absolute',
									top: topPosition + shift < this.dim.height ? topPosition - shift > 0 ? topPosition : shift : this.dim.height - shift,
									left: leftPosition + shift < this.dim.width ? leftPosition - shift > 0 ? leftPosition : shift : this.dim.width - shift,								
									width: this.options.markerSize,
									height: this.options.markerSize,
									marginLeft: - shift,
									marginTop: - shift,
									zIndex: 100,
									display: 'block',
									cursor: 'pointer'
								}
							
							return $('<span />').addClass(this.classes.marker).css(styleMarker);
		
						},
		
		showTooltip:	function(e) {
		
							var $marker = $(e.currentTarget),
								data = $marker.data('data'),
								$tooltip = $marker.data('tooltip');
							
							if ($tooltip && $tooltip.length) {
								
								var delayRemoval = $tooltip.data('delayRemoval');
								if (delayRemoval) clearTimeout(delayRemoval);
								return;
								
							}
							
							var $tooltip = this.getTooltip(data).appendTo(this.$markers).css('opacity', 0);
							
							$marker.data('tooltip', $tooltip)
							$tooltip.on('mouseover', $.proxy(this.tooltipHover, this));
							$tooltip.on('mouseout', $.proxy(this.tooltipOut, this));
								
							$tooltip.data('ready').done($.proxy(function() {
								
								$marker.addClass('active').data('active', true);
								$tooltip.data('marker', $marker);
								
								var $arrow = $tooltip.find('.' + this.classes.tooltipArrow),
									positionTooltip = this.getTooltipPos($marker, $tooltip),
									positionArrow = this.getArrowPos($marker, $tooltip, $arrow, positionTooltip);
								
								$tooltip.css(positionTooltip.position);
								$arrow.css(positionArrow);
								
								switch (positionTooltip.type) {
								
									case 'tc': margin = {marginTop: this.options.marginTooltip}; break;
									case 'bc': margin = {marginTop: -this.options.marginTooltip}; break;
									case 'lc': margin = {marginRight: -this.options.marginTooltip}; break;
									case 'rc': margin = {marginLeft: -this.options.marginTooltip}; break;
									case 'tl': margin = {marginTop: this.options.marginTooltip}; break;
									case 'tr': margin = {marginTop: this.options.marginTooltip}; break;
									case 'bl': margin = {marginTop: -this.options.marginTooltip}; break;
									case 'br': margin = {marginTop: -this.options.marginTooltip}; break;
								
								}
								
								setTimeout($.proxy(function() {
									
									$tooltip.css(margin).stop().animate({
										margin: 0,
										opacity: 1
									}, this.options.speedShowTooltip);
								
								}, this));
								
							}, this));
		
						},
		
		getTooltip:		function(data) {
		
							var styleArrow = {
									position: 'absolute',
									zIndex: 999,
									display: 'block'
								},
								styleTooltipWrap = {
									position: 'absolute',
									top: 0,
									left: 0,
									zIndex: 999
								};
							
							var $arrow = $('<span />').addClass(this.classes.tooltipArrow).css(styleArrow),
								$clear = $('<div />').css('clear', 'both'),
								$img = data.image ? $('<img />', {src: data.image}).addClass(this.classes.tooltipImage) : null,
								$text = $('<div />').html(data.text).prepend($img).append($clear.clone()),
								$link = data.link ? $('<a />', {href: data.link, target: '_blank', text: data.linkText || 'link'}).addClass(this.classes.tooltipLink).css('display', 'block') : null,
								
								$tooltip = $('<div />').addClass(this.classes.tooltip).addClass(this.checkBoxShadow).append($text).append($link).append($clear),
								$tooltipWrap = $('<div />').addClass(this.classes.tooltipWrap).css(styleTooltipWrap).append($arrow).append($tooltip),
								deferred = $.Deferred(),
								resolve = function() { deferred.resolve() }
							
							if (data.image) this.waitImage(data.image, resolve); 
							else resolve();
							
							return $tooltipWrap.data('ready', deferred);
		
						},
		
		hideTooltip:	function($marker) {
		
							var $tooltip = $marker.data('tooltip');
							
							if (!$tooltip) return;
							
							$tooltip.data('delayRemoval', setTimeout($.proxy(function() {
								
								var deferred = $tooltip.data('ready');
								if (deferred) deferred.reject();
								
								$tooltip.remove();
								$marker.removeData('tooltip').removeClass('active').data('active', false);
								$('html').off('touchend.' + this.id + $marker.index()).off('touchmove');
							
							}, this), 50));
							
						},
		
		markerOut:		function(e) {
		
							var $marker = $(e.currentTarget);
							this.hideTooltip($marker);
		
						},
		
		tooltipHover:	function(e) {
		
							var $tooltip = $(e.currentTarget),
								delayRemoval = $tooltip.data('delayRemoval');
							
							if (delayRemoval) clearTimeout(delayRemoval);
		
						},
		
		tooltipOut:		function(e) {
		
							var $tooltip = $(e.currentTarget),
								$marker = $tooltip.data('marker');
							
							this.hideTooltip($marker);
		
						},
		
		getTooltipPos:	function($marker, $tooltip) {
		
							if (!$marker || !$marker.length || !$tooltip || !$tooltip.length) return;
							
							var offset = $marker.offset(),
								markerPosition = $marker.position(),
								window = {								
									width: document.body.scrollWidth,
									height: document.body.scrollHeight
								},
								markerEdges = {
									top: offset.top,
									right: window.width - offset.left - this.options.markerSize,
									left: offset.left,
									bottom: window.height - offset.top -this.options.markerSize
								},
								tooltipDim = {
									width: $tooltip.outerWidth(),
									height: $tooltip.outerHeight()
								},
								test = {									
									top: $.proxy(function() { return this.checkPos(tooltipDim.height + this.options.marginTooltip, markerEdges.top) }, this),
									topHalf: $.proxy(function() { return this.checkPosHalf(tooltipDim.height, markerEdges.top) }, this),
									right: $.proxy(function() { return this.checkPos(tooltipDim.width + this.options.marginTooltip, markerEdges.right) }, this),
									rightHalf: $.proxy(function() { return this.checkPosHalf(tooltipDim.width, markerEdges.right) }, this),
									bottom: $.proxy(function() { return this.checkPos(tooltipDim.height  + this.options.marginTooltip, markerEdges.bottom) }, this),
									bottomHalf: $.proxy(function() { return this.checkPosHalf(tooltipDim.height, markerEdges.bottom) }, this),
									left: $.proxy(function() { return this.checkPos(tooltipDim.width + this.options.marginTooltip, markerEdges.left) }, this),
									leftHalf: $.proxy(function() { return this.checkPosHalf(tooltipDim.width, markerEdges.left) }, this)
								},
								getPosition = {
									top: $.proxy(function() { return markerPosition.top - this.options.marginTooltip - tooltipDim.height - this.options.markerSize / 2 }, this),
									topHalf: $.proxy(function() { return markerPosition.top - tooltipDim.height / 2 }, this),
									right: $.proxy(function() { return markerPosition.left + this.options.marginTooltip + this.options.markerSize - this.options.markerSize / 2 }, this),
									rightSnap: $.proxy(function() { return markerEdges.right === 0 ? this.dim.width - tooltipDim.width : markerPosition.left + this.options.markerSize / 2 + markerEdges.right - this.options.marginTooltip - tooltipDim.width }, this),
									bottom: $.proxy(function() { return markerPosition.top + this.options.marginTooltip + this.options.markerSize / 2 }, this),
									left: $.proxy(function() { return markerPosition.left - this.options.marginTooltip - tooltipDim.width - this.options.markerSize / 2 }, this),
									leftHalf: $.proxy(function() { return markerPosition.left - tooltipDim.width / 2 }, this),
									leftSnap: $.proxy(function() { return markerEdges.left === 0 ? 0 : markerPosition.left - this.options.markerSize / 2 - markerEdges.left + this.options.marginTooltip }, this)
								}
							
							if 		(test.top() && test.rightHalf() && test.leftHalf()) 	return {type: 'tc', position: {top: getPosition.top(), left: getPosition.leftHalf()}};
							else if	(test.bottom() && test.rightHalf() && test.leftHalf())	return {type: 'bc', position: {top: getPosition.bottom(), left: getPosition.leftHalf()}};
							else if	(test.left() && test.topHalf() && test.bottomHalf()) 	return {type: 'lc', position: {top: getPosition.topHalf(), left: getPosition.left()}};
							else if	(test.right() && test.topHalf() && test.bottomHalf()) 	return {type: 'rc', position: {top: getPosition.topHalf(), left: getPosition.right()}};
							else if	(test.top() && !test.leftHalf()) 						return {type: 'tl', position: {top: getPosition.top(), left: getPosition.leftSnap()}};
							else if	(test.top() && !test.rightHalf()) 						return {type: 'tr', position: {top: getPosition.top(), left: getPosition.rightSnap()}};
							else if	(test.bottom() && !test.leftHalf()) 					return {type: 'bl', position: {top: getPosition.bottom(), left: getPosition.leftSnap()}};
							else if	(test.bottom() && !test.rightHalf()) 					return {type: 'br', position: {top: getPosition.bottom(), left: getPosition.rightSnap()}};
							else 															return {type: 'bc', position: {top: getPosition.bottom(), left: getPosition.leftHalf()}};
							
						},
		
		checkPos:		function(side, edge) {
		
							if (side <= edge) return true;
							else return false;
		
						},
		
		checkPosHalf:	function(side, edge) {
		
							if ((side / 2) <= edge) return true;
							else return false;
		
						},
		
		getArrowPos:	function ($marker, $tooltip, $arrow, positionTooltip) {
		
							var position,
								markerPosition = $marker.position(),
								tooltipPosition = positionTooltip.position,
								arrowDim = {
									width: $arrow.width(),
									height: $arrow.height()
								},
								border = {
									top: $.proxy(function() { return parseFloat($tooltip.find('.' + this.classes.tooltip).css('borderTopWidth').replace('px', '')) || 0 }, this),
									right: $.proxy(function() { return parseFloat($tooltip.find('.' + this.classes.tooltip).css('borderRightWidth').replace('px', '')) || 0 }, this),
									bottom: $.proxy(function() { return parseFloat($tooltip.find('.' + this.classes.tooltip).css('borderBottomWidth').replace('px', '')) || 0 }, this),
									left: $.proxy(function() { return parseFloat($tooltip.find('.' + this.classes.tooltip).css('borderLeftWidth').replace('px', '')) || 0 }, this)
								}
							
							switch (positionTooltip.type) {
							
								case 'tc': position = {bottom: -arrowDim.height + border.bottom(), left: '50%', marginLeft: -arrowDim.width / 2, backgroundPosition: '0px -' + arrowDim.height * 0 + 'px'}; break;
								case 'bc': position = {top: -arrowDim.height + border.top(), left: '50%', marginLeft: -arrowDim.width / 2, backgroundPosition: '0px -' + arrowDim.height * 2 + 'px'}; break;
								case 'lc': position = {top: '50%', right: -arrowDim.width + border.right(), marginTop: -arrowDim.height / 2, backgroundPosition: '0px -' + arrowDim.height * 3 + 'px'}; break;
								case 'rc': position = {top: '50%', left: -arrowDim.width + border.left(), marginTop: -arrowDim.height / 2, backgroundPosition: '0px -' + arrowDim.height * 1 + 'px'}; break;
								case 'tl': position = {bottom: -arrowDim.height + border.bottom(), left: markerPosition.left - tooltipPosition.left, marginLeft: -arrowDim.width / 2, backgroundPosition: '0px -' + arrowDim.height * 0 + 'px'}; break;
								case 'tr': position = {bottom: -arrowDim.height + border.bottom(), left: markerPosition.left - tooltipPosition.left, marginLeft: -arrowDim.width / 2, backgroundPosition: '0px -' + arrowDim.height * 0 + 'px'}; break;
								case 'bl': position = {top: -arrowDim.height + border.top(), left: markerPosition.left - tooltipPosition.left, marginLeft: -arrowDim.width / 2, backgroundPosition: '0px -' + arrowDim.height * 2 + 'px'}; break;
								case 'br': position = {top: -arrowDim.height + border.top(), left: markerPosition.left - tooltipPosition.left, marginLeft: -arrowDim.width / 2, backgroundPosition: '0px -' + arrowDim.height * 2 + 'px'}; break;
							
							}
							
							return position;
		
						}
		
	});  

	
//	=========================================================================================================		
//		jQuery extend
//	=========================================================================================================


	$.fn[pluginName] = function (options) {
	
		return this.each(function () {
			
			if (!$.data(this, 'plugin_' + pluginName)) {
			
				$.data(this, 'plugin_' + pluginName, new Plugin(this, options));
				
			}
			
		});
		
	}

})(jQuery, window, document);